import java.io.InputStream;
import java.io.FileInputStream;
import java.io.IOException;

public class EBNFScanner implements EBNFTokens {
  public EBNFScanner (InputStream in) {
    this.in = in;
  }
  private InputStream in;
  private final char EOFChar = (char)-1;
  public int token = BAD;
  private String chrs = "";
  private char chr = ' ';
  private void nextChr() {
    try {
      chr = (char)in.read();
    } catch (IOException e) {
      throw new Error("Erreur d'E/S");
    }
  }
  public String representation() {
    switch (token) {
      case BAD   : return "<bad> ";
      case EOF   : return "<eof> ";
      case IDENT   : return "IDENT("+chrs+") ";
      case LITERAL : return "LITERAL("+chrs+") ";
      case LPAREN  : return "LPAREN ";
      case RPAREN  : return "RPAREN ";
      case LBRACK  : return "LBRACK ";
      case RBRACK  : return "RBRACK ";
      case LBRACE  : return "LBRACE ";
      case RBRACE  : return "RBRACE ";
      case BAR   : return "BAR ";
      case EQL   : return "EQL ";
      case PERIOD  : return "PERIOD ";
      default    : throw new Error("Yapa ce token !");
    }
  }
  public void close() {
    try {
      in.close();
    } catch (IOException e) {
      throw new Error("Erreur d'E/S");
    }
  }
  public void nextToken() {
    while (chr <= ' ') {
      nextChr();
    }
    if (Character.isLetter(chr)) {
      StringBuffer buf = new StringBuffer();
      buf.append(chr);
      nextChr();
      while (Character.isLetterOrDigit(chr)) {
        buf.append(chr);
        nextChr();
      }
      token = IDENT;
      chrs = buf.toString();
    } else {
      switch (chr) {
        case '\u0022':
          nextChr();
          StringBuffer buf = new StringBuffer();
          while (chr != '\u0022' && chr != EOFChar && chr >= ' ') {
            if (chr == '\\') {
              nextChr();
            }
            buf.append(chr);
            nextChr();
          }
          if (chr == '\u0022') {
            nextChr();
            token = LITERAL;
            chrs = buf.toString();
          } else {
            throw new Error("Chane non-termine");
          }
          break;
        case '(': token = LPAREN; nextChr(); break;
        case ')': token = RPAREN; nextChr(); break;
        case '{': token = LBRACE; nextChr(); break;
        case '}': token = RBRACE; nextChr(); break;
        case '[': token = LBRACK; nextChr(); break;
        case ']': token = RBRACK; nextChr(); break;
        case '|': token = BAR; nextChr(); break;
        case '=': token = EQL; nextChr(); break;
        case '.': token = PERIOD; nextChr(); break;
        case EOFChar: token = EOF; break;
        default: token = BAD; nextChr(); break;
      }
    }
  }
}