import java.io.InputStream;
import java.io.FileInputStream;

object EBNFTokens extends Enumeration {
  type Token = Value;
  val BAD = Value("<bad>");
  val EOF = Value("<eof>");
  val IDENT = Value("ident");
  val LITERAL = Value("literal");
  val LPAREN = Value("("); val RPAREN = Value(")");
  val LBRACK = Value("["); val RBRACK = Value("]");
  val LBRACE = Value("{"); val RBRACE = Value("}");
  val BAR = Value("|");
  val EQL = Value("="); 
  val PERIOD = Value(".");
}

class EBNFScanner (in: InputStream) {
  import EBNFTokens._;
  private val EOFChar =
    (-1).asInstanceOf[Char];
  var token: Token = BAD;
  var chars: String = "";
  private var char = ' ';
  private def nextChar =
  	char = in.read().asInstanceOf[Char];
  def representation: String = token match {
    case IDENT | LITERAL =>
      token.toString() + "(" + chars + ") ";
    case _ => token.toString() + " ";
  }
  def close = in.close();
  def nextToken = {
    while (char <= ' ') nextChar;
    if (Character.isLetter(char)) {
      val buf = new StringBuffer();
      buf.append(char); nextChar;
      while (Character.isLetterOrDigit(char)) {
        buf.append(char); nextChar;
      }
      token = IDENT;
      chars = buf.toString();
    } else char match {
      case '\u0022' => {
        nextChar;
        val buf = new StringBuffer();
        while (char != '\u0022' && 
               char != EOFChar && 
               char >= ' ') {
          if (char == '\\') nextChar;
          buf.append(char); nextChar;
        }
        if (char == '\u0022') {
          nextChar;
          token = LITERAL;
          chars = buf.toString();
        } else
          error("Chaîne non-terminée");
      }
      case '(' => token = LPAREN; nextChar;
      case ')' => token = RPAREN; nextChar;
      case '[' => token = LBRACK; nextChar;
      case ']' => token = RBRACK; nextChar;
      case '{' => token = LBRACE; nextChar;
      case '}' => token = RBRACE; nextChar;
      case '|' => token = BAR; nextChar;
      case '=' => token = EQL; nextChar;
      case '.' => token = PERIOD; nextChar;
      case EOFChar => token = EOF;
      case _ => token = BAD; nextChar;
    }
  }
}

object EBNFScannerTest {
  import EBNFTokens._;
  def main (args: Array[String]) = {
    val scan = new EBNFScanner(new FileInputStream(args(0)));
    scan.nextToken;
    while (scan.token != EOF) {
      Console.print(scan.representation);
      scan.nextToken;
    }
    Console.println(scan.representation);
    scan.close;
  }
}